/*---------------------------------------------------------------------------------
	$Id: template.c,v 1.3 2008-03-17 18:15:38 web1 Exp $

	Simple ARM7 stub (sends RTC, TSC, and X/Y data to the ARM 9)

	$Log: template.c,v $
	Revision 1.3  2008-03-17 18:15:38  web1
	Zufallszahlen richtig initialisiert. Wechsel der Spielerfarbe funktioniert jetzt

	Revision 1.2  2007-08-16 20:47:26  web1
	Jede Menge neue Features
	wahrscheinlich Version 0.3

	Revision 1.1  2006-05-23 21:59:07  web1
	Neue alte Version von DSChess jetzt mit besserem Template
	Box angefangen, Zeit auf Timer umgestellt wegen Inkompatibilitt mit Dualis
	
	Revision 1.2  2005/09/07 20:06:06  wntrmute
	updated for latest libnds changes

	Revision 1.8  2005/08/03 05:13:16  wntrmute
	corrected sound code


---------------------------------------------------------------------------------*/
#include <nds.h>
//#include <nds/bios.h>
//#include <nds/arm7/touch.h>
//#include <nds/arm7/clock.h>

//---------------------------------------------------------------------------------
void startSound(int sampleRate, const void* data, u32 bytes, u8 channel, u8 vol,  u8 pan, u8 format) {
//---------------------------------------------------------------------------------
	SCHANNEL_TIMER(channel)  = SOUND_FREQ(sampleRate);
	SCHANNEL_SOURCE(channel) = (u32)data;
	SCHANNEL_LENGTH(channel) = bytes >> 2 ;
	SCHANNEL_CR(channel)     = SCHANNEL_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(vol) | SOUND_PAN(pan) | (format==1?SOUND_8BIT:SOUND_16BIT);
}


//---------------------------------------------------------------------------------
s32 getFreeSoundChannel() {
//---------------------------------------------------------------------------------
	int i;
	for (i=0; i<16; i++) {
		if ( (SCHANNEL_CR(i) & SCHANNEL_ENABLE) == 0 ) return i;
	}
	return -1;
}

static int nx, ny;
static bool updateTouch(){
	int c = 7;
	int i,j;
	int x[c];
	int y[c];
	int temp;

	for (i=0; i<c; i++){
    	if (!(REG_KEYXY & (1<<6))){
			x[i] = touchRead(TSC_MEASURE_X);
    		y[i] = touchRead(TSC_MEASURE_Y);
    	}
    	else return false;
	}
	//Bubblesort
	for (i=0;i<c-1;i++){
		for (j=0;j<c-1;j++){
			if (x[j]>x[j+1]){
				temp = x[j];x[j] = x[j+1];x[j+1] = temp;
			}
			if (y[j]>y[j+1]){
				temp = y[j];y[j] = y[j+1];y[j+1] = temp;
			}
		}
	}
	//Take the median
	nx = x[(c-1)/2];
	ny = y[(c-1)/2];
	return true;
}

#define ABS(x) ((x) < 0) ? -(x) : (x)
static bool touchInit = false;
static s32 xscale, yscale;
static s32 xoffset, yoffset;
void setTouchIPC(int x, int y){
	if ( !touchInit ) {
		xscale = ((PersonalData->calX2px - PersonalData->calX1px) << 19) / ((PersonalData->calX2) - (PersonalData->calX1));
		yscale = ((PersonalData->calY2px - PersonalData->calY1px) << 19) / ((PersonalData->calY2) - (PersonalData->calY1));

		xoffset = (PersonalData->calX1) * xscale - (PersonalData->calX1px << 19);
		yoffset = (PersonalData->calY1) * yscale - (PersonalData->calY1px << 19);

		touchInit = true;
	}
	s16 px = ( x * xscale - xoffset + xscale/2 ) >>19;
	s16 py = ( y * yscale - yoffset + yscale/2 ) >>19;

	if ( px < 0) px = 0;
	if ( py < 0) py = 0;
	if ( px > (SCREEN_WIDTH -1)) px = SCREEN_WIDTH -1;
	if ( py > (SCREEN_HEIGHT -1)) py = SCREEN_HEIGHT -1;
	IPC->touchX = x;
	IPC->touchY = y;
	IPC->touchXpx = px;
	IPC->touchYpx = py;
}

//---------------------------------------------------------------------------------
void VblankHandler(void) {
//---------------------------------------------------------------------------------
	static int heartbeat = 0;
	static const int savePad = 3;
	static int oldX[3], oldY[3], oldKeys;
	static int firsttouch;

	uint16 but=0, z1=0, z2=0, batt=0, aux=0;
	int t1=0, t2=0;
	uint32 temp=0;
	//uint8 ct[sizeof(IPC->curtime)];
	uint8 ct[8];
	u32 i;
	bool cleanread=false;

	// Update the heartbeat
	heartbeat++;

	// Read the touch screen

	but = REG_KEYXY;

	if (!(but & (1<<6))) {
		cleanread = updateTouch();
		if (((oldKeys & (1<<6)) != 0)) firsttouch = heartbeat;
	}

	z1 = touchRead(TSC_MEASURE_Z1);
	z2 = touchRead(TSC_MEASURE_Z2);


	batt = touchRead(TSC_MEASURE_BATTERY);
	aux  = touchRead(TSC_MEASURE_AUX);

	// Read the time
	rtcGetTime((uint8 *)ct);
	BCDToInteger((uint8 *)&(ct[1]), 7);

	// Read the temperature
	temp = touchReadTemperature(&t1, &t2);

	// Update the IPC struct
	//IPC->heartbeat	= heartbeat;
	IPC->touchZ1		= z1;
	IPC->touchZ2		= z2;
	IPC->battery		= batt;
	IPC->aux			= aux;

	//gerade losgelassen
	if (((oldKeys & (1<<6)) == 0) && (!cleanread)){
		setTouchIPC (oldX[0], oldY[0]);
		IPC->buttons  = REG_KEYXY & ~(1<<6);
	}
	else if (cleanread){
		for (i=0; i<savePad - 1; i++){
			oldX[i]=oldX[i+1];
			oldY[i]=oldY[i+1];
		}
		oldX[savePad-1] = nx;
		oldY[savePad-1] = ny;
		setTouchIPC (oldX[savePad - 2], oldY[savePad - 2]);
	}
	if (ABS(heartbeat - firsttouch) < 4){
		//we have to wait
		IPC->buttons = but | REG_KEYXY | (1<<6);
	}
	else IPC->buttons = but | REG_KEYXY;
	oldKeys = but | REG_KEYXY;

	for(i=0; i<sizeof(ct); i++) {
		IPC->time.curtime[i] = ct[i];
	}

	IPC->temperature = temp;
	IPC->tdiode1 = t1;
	IPC->tdiode2 = t2;


	//sound code  :)
	TransferSound *snd = IPC->soundData;
	IPC->soundData = 0;

	if (0 != snd) {

		for (i=0; i<snd->count; i++) {
			s32 chan = getFreeSoundChannel();

			if (chan >= 0) {
				startSound(snd->data[i].rate, snd->data[i].data, snd->data[i].len, chan, snd->data[i].vol, snd->data[i].pan, snd->data[i].format);
			}
		}
	}

}

//---------------------------------------------------------------------------------
int main(int argc, char ** argv) {
//---------------------------------------------------------------------------------

	// Reset the clock if needed
	readUserSettings();

	//enable sound
	powerON(POWER_SOUND);
	writePowerManagement(PM_CONTROL_REG, ( readPowerManagement(PM_CONTROL_REG) & ~PM_SOUND_MUTE ) | PM_SOUND_AMP );
	//powerON(POWER_UNKNOWN);
	SOUND_CR = SOUND_ENABLE | SOUND_VOL(0x7F);
	IPC->soundData = 0;

	irqInit();
	initClockIRQ();
	SetYtrigger(80);
	
	irqSet(IRQ_VBLANK, VblankHandler);
	irqEnable(IRQ_VBLANK);

	// Keep the ARM7 out of main RAM
	while (1) swiWaitForVBlank();
}


