#include "NdsMenu.h"
#include "nds.h"
#include "fonts.h"
#include <stdio.h>
#include <string.h>
#include "click1_raw.h"
#include "click2_raw.h"
#include <nds/arm9/sound.h>		// sound functions

#define BGCOLOR (RGB15(31,31,31)|(1<<15))
#define TEXT_COLOR (RGB15(0,0,0)|(1<<15))
#define SELECT_COLOR (RGB15(31,0,0)|(1<<15))
#define ACTIVE_COLOR (RGB15(0,0,0)|(1<<15))
#define INACTIVE_COLOR (RGB15(20,20,20)|(1<<15))
#define LINEHEIGHT 12

TransferSoundData click1 = {
		click1_raw,		/* Sample address */
		click1_raw_size,	/* Sample length */
		44100,				/* Sample rate */
		50,				/* Volume */
		64,					/* panning */
		2 					/* format */
	};
TransferSoundData click2 = {
		click2_raw,		/* Sample address */
		click2_raw_size,	/* Sample length */
		44100,				/* Sample rate */
		50,				/* Volume */
		64,					/* panning */
		2 					/* format */
	};

Menu::Menu(char * _name):name(_name){
	posx = 30;
	posy = 30;
}

void Menu::addMenuItem(menuItem * item){
	item->parent = this;
	subItems[entryCount++] = item;
}

void Menu::drawItem (int item, bool selected){
	char buf[256];
	uint16 color = TEXT_COLOR;
	if (selected) color = SELECT_COLOR;
	if (!subItems[item]->isActive()) color = INACTIVE_COLOR;
	sprintf(buf, "%s", (*subItems[item]).getText());
	clearRect (BG_GFX_SUB, posx, posy+item*LINEHEIGHT+LINEHEIGHT, SCREEN_WIDTH, 
		posy+item*LINEHEIGHT+2*LINEHEIGHT, BGCOLOR);
	printBigCenterString(BG_GFX_SUB, posy+item*LINEHEIGHT+LINEHEIGHT, buf, color);
}

void Menu::redraw(int selItem){
	char buf[256];
	//clear Screen
	swiWaitForVBlank();
	clearScreen(BG_GFX_SUB, BGCOLOR);
	sprintf(buf, "-----%s-----", name);
	printBigCenterString(BG_GFX_SUB, posy, buf, TEXT_COLOR);
	for (int i=0; i<entryCount; i++){
		if (i == selItem) drawItem(i, true);
		else drawItem(i, false);
	}
}

int Menu::buttonAHandler(int selItem){
	return (*subItems[selItem]).execute(KEY_A);
}

int Menu::display(){
	setbgColor(BGCOLOR);
	int selItem = 0;
	while (selItem < entryCount && !(subItems[selItem]->isActive())) selItem++;
	//All entries are inactive
	if (selItem == entryCount) return 0;
	redraw(selItem);
	uint16 keysPressed = ~(REG_KEYINPUT);
	uint16 lastPressed = ~(REG_KEYINPUT);
	while (true){
		lastPressed = keysPressed;
		keysPressed = ~(REG_KEYINPUT);
		if (!(lastPressed & KEY_UP) && (keysPressed & KEY_UP)){
			int newitem = selItem-1;
			while (newitem >= 0 && !(subItems[newitem]->isActive())) newitem--;
			if (newitem >= 0){
				playSound(&click1);
				swiWaitForVBlank();
				drawItem (selItem, false);
				selItem = newitem;
				drawItem (selItem, true);
			}
		}
		else if (!(lastPressed & KEY_DOWN) && (keysPressed & KEY_DOWN)){
			int newitem = selItem+1;
			while (newitem < entryCount && !(subItems[newitem]->isActive())) newitem++;
			if (newitem < entryCount){
				playSound(&click1);
				swiWaitForVBlank();
				drawItem (selItem, false);
				selItem = newitem;
				drawItem (selItem, true);
			}
		}
		else if (!(lastPressed & KEY_RIGHT) && (keysPressed & KEY_RIGHT) && 
		(*subItems[selItem]).hasSelection()){
			playSound(&click2);
			(*subItems[selItem]).execute(keysPressed);
			drawItem (selItem, true);
		}
		else if (!(lastPressed & KEY_LEFT) && (keysPressed & KEY_LEFT) && 
		(*subItems[selItem]).hasSelection()){
			playSound(&click2);
			(*subItems[selItem]).execute(keysPressed);
			drawItem (selItem, true);
		}
		else if (!(lastPressed & KEY_B) && (keysPressed & KEY_B)){
			playSound(&click2);
			return 0;
		}
		else if (!(lastPressed & KEY_A) && (keysPressed & KEY_A)){
			playSound(&click2);
			int retstate = buttonAHandler(selItem);
			if (retstate == 1) return 0;
			if (retstate == 2) return 2;
			redraw (selItem);
			keysPressed = ~(REG_KEYINPUT); //HACK to avoid exit through B
		}
		else if(!(lastPressed & KEY_START) && (keysPressed & KEY_START)){
			playSound(&click2);
			return 2;
		}
		swiWaitForVBlank();
	}
}

void subMenuItem::setSubMenu(Menu * _sMenu){
	sMenu = _sMenu;
}

int subMenuItem::execute(uint16 keys){
	if (keys & KEY_A) return sMenu->display();
	else return 0;
}

void selectionMenu::drawItem(int item, bool selected){
	char buf[256];
	clearRect (BG_GFX_SUB, posx, posy+item*LINEHEIGHT+LINEHEIGHT, 
		SCREEN_WIDTH, posy+item*LINEHEIGHT+2*LINEHEIGHT, BGCOLOR);
	uint16 color = TEXT_COLOR;
	if (selected) color = SELECT_COLOR;
	if (this->subItems[item] == choosen){
		sprintf(buf, ">%s", (*subItems[item]).getText());
		printBigCenterString(BG_GFX_SUB, posy+item*LINEHEIGHT+LINEHEIGHT, buf, color);
	}
	else{
		sprintf(buf, " %s", (*subItems[item]).getText());
		printBigCenterString(BG_GFX_SUB, posy+item*LINEHEIGHT+LINEHEIGHT, buf, color);	
	}
}

int selectionMenu::buttonAHandler(int selItem){
	//TERRIBLE HACK!!!!
	//Todo: Change this!!
	if (subItems[selItem]->getText() != "Back..."){
		choosen = subItems[selItem];
		return 0;
	}
	else return subItems[selItem]->execute(KEY_A);
}

void SimpleSelectionMenuItem::addSelectionItem(char * selItem){
	selItems[entryCount++] = selItem;
}

char* SimpleSelectionMenuItem::getText(){
	sprintf(buf, "%s <%s>", name, selItems[selectedItem]);
	return buf;
}

int SimpleSelectionMenuItem::execute(const uint16 keys){
	if (keys & KEY_A) selectedItem++;
	else if (keys & KEY_RIGHT) selectedItem++;
	else if (keys & KEY_LEFT) selectedItem--;
	selectedItem = selectedItem % entryCount;
	return 0;
}
