#include "nds.h"
#include "smallfont.h"
#include "bigfont.h"
#include "fonts.h"
#include "string.h"
#include "stdio.h"
#include "stdarg.h"

static uint16 bgColor;

/**
 * Sets the backgroundcolor for the upcoming chars
 */ 
void setbgColor(uint16 _bgColor){
	bgColor = _bgColor;
}

void clearRect (uint16* GFX, int x1, int y1, int x2, int y2, const uint16 color){
	const int l = (x2-x1);
	for (int j=y1; j<y2; j++){
		swiCopy(&color, &(GFX[j*SCREEN_WIDTH + x1]), l|COPY_MODE_FILL);
		//memset(&(GFX[j*SCREEN_WIDTH + x1]), color, l*2);
/*		for (int i=x1; i<x2; i++){
			GFX[j*SCREEN_WIDTH + i] = color;
		}*/
	}
}

void clearScreen (uint16* GFX, const uint16 color){
	const uint16 f = color|1<<15;
	swiCopy(&f, GFX, (SCREEN_WIDTH * SCREEN_HEIGHT)|COPY_MODE_FILL);
	//memset(GFX, f, SCREEN_WIDTH * SCREEN_HEIGHT*2);
	
	//for (int i=0;i<SCREEN_WIDTH * SCREEN_HEIGHT;i++) GFX[i]=color|1<<15;
}

void printChar (uint16* GFX, int x, int y, char c, unsigned int color){
	unsigned short CharX = smallset.Chars[(unsigned short)c].x;
	unsigned short CharY = smallset.Chars[(unsigned short)c].y;
	unsigned short Width = smallset.Chars[(unsigned short)c].Width;
	unsigned short Height = smallset.Chars[(unsigned short)c].Height;
	unsigned short XOffset = smallset.Chars[(unsigned short)c].XOffset;
	unsigned short YOffset = smallset.Chars[(unsigned short)c].YOffset;
	for (int i=0;i<Height;i++){
		for (int j=0;j<Width;j++){
			if (smallfont_gfx[(CharY + i)*smallset.Width + CharX+j] != 0){
				uint16 p = smallfont_gfx[(CharY + i)*smallset.Width + CharX+j] & color;
				GFX[(y + i + YOffset)*SCREEN_WIDTH + x + j + XOffset] = p|(1<<15);
			}
		}
	}
}


/**
 * Returns the width of the string in pixel
 */
int getStringWidth (const char *str) {
	int l = 0;
	for (unsigned int i=0; i<strlen(str); i++){
		if (str[i] == ' ') l += smallset.Base;
		else l += smallset.Chars[(unsigned short)str[i]].XAdvance;
	}
	return l;
}

void printString(uint16* GFX, int x, int y, const char * str, unsigned int color, 
	bool clear){
	
	int cursor = x;
	int ccursor = x;
	int line = y;
	for (unsigned int i=0;i<strlen(str);i++){
		if (str[i] == '\n'){
			line += smallset.LineHeight;
			cursor = x;
			ccursor = x;
			continue;
		}
		if (clear && ccursor < SCREEN_WIDTH){
			int end = ccursor + FONTHEIGHT;
			if (end > SCREEN_WIDTH) end = SCREEN_WIDTH;
			clearRect(GFX, ccursor, line, end, line + smallset.LineHeight, bgColor);
		}
		ccursor += FONTHEIGHT;
		if (str[i] == ' '){
			cursor += smallset.Base;
			if (cursor > SCREEN_WIDTH){
				line += smallset.LineHeight;
				cursor = x;
				ccursor = x+8;
			}
			continue;
		}
		else if (cursor + smallset.Chars[(unsigned short)str[i]].XAdvance > SCREEN_WIDTH){
			line += smallset.LineHeight;
			cursor = x;
			ccursor = x+8;
		}
		if (str[i] == '#') {
			//HACK, because # is not part of the font
			printChar(GFX, cursor, line-1, '+', color);
			printChar(GFX, cursor, line+1, '+', color);
		}
		else printChar(GFX, cursor, line, str[i], color);
		cursor += smallset.Chars[(unsigned short)str[i]].XAdvance;
	}
}

void printfs(const char *format, ...){
    static int row, col;
    static int savedX, savedY;
	char buffer[256];
	va_list args;
	va_start( args, format );
    vsprintf(buffer, format, args);
    va_end(args);
    int i = 1;
    if ( buffer[i-1] == '\x1b' && buffer[i] == '[' ) {
		bool escaping = true;
		char *escapeseq	= &buffer[i];
		int escapelen = 0;
		do {
			char chr = buffer[i+1];
			i++; escapelen++;
			int parameter;
			switch (chr) {
				case 'H':
					sscanf(escapeseq,"[%d;%dH", &row, &col);
					escaping = false;
					break;
				case 'f':
					sscanf(escapeseq,"[%d;%df", &row, &col);
					escaping = false;
					break;
				case 'A':
					sscanf(escapeseq,"[%dA", &parameter);
					row =  (row - parameter) < 0 ? 0 : row - parameter;
					escaping = false;
					break;
				case 'B':
					sscanf(escapeseq,"[%dB", &parameter);
					//row =  (row + parameter) > CONSOLE_HEIGHT - 1 ? CONSOLE_HEIGHT - 1 : row + parameter;
					escaping = false;
					break;
				case 'C':
					sscanf(escapeseq,"[%dC", &parameter);
					//col =  (col + parameter) > CONSOLE_WIDTH - 1 ? CONSOLE_WIDTH - 1 : col + parameter;
					escaping = false;
					break;
				case 'D':
					sscanf(escapeseq,"[%dC", &parameter);
					col =  (col - parameter) < 0 ? 0 : col - parameter;
					escaping = false;
					break;
				case 'K':
					escaping = false;
					break;
				case 's':
					savedX = col;
					savedY = row;
					escaping = false;
					break;
				case 'u':
					col = savedX;
					row = savedY;
					escaping = false;
					break;
				case 'J':
					if ( escapeseq[escapelen-2] == '2') {
						//consoleCls();
					}
					escaping = false;
				break;
			}
		} while (escaping);
	}
    printString(BG_GFX_SUB, col * smallset.Base, row * smallset.LineHeight, &buffer[i+1], RGB15(30, 30, 30));
}


/**
 * Returns the width of the string in pixel (big font)
 */
int getBigStringWidth (const char *str) {
	int l = 0;
	for (unsigned int i=0; i<strlen(str); i++){
		if (str[i] == ' ') l += bigset.Base;
		else l += bigset.Chars[(unsigned short)str[i]].XAdvance;
	}
	return l;
}

void printBigChar (uint16* GFX, int x, int y, char c, unsigned int color){
	unsigned short CharX = bigset.Chars[(unsigned short)c].x;
	unsigned short CharY = bigset.Chars[(unsigned short)c].y;
	unsigned short Width = bigset.Chars[(unsigned short)c].Width;
	unsigned short Height = bigset.Chars[(unsigned short)c].Height;
	unsigned short XOffset = bigset.Chars[(unsigned short)c].XOffset;
	unsigned short YOffset = bigset.Chars[(unsigned short)c].YOffset;
	for (int i=0;i<Height;i++){
		for (int j=0;j<Width;j++){
			if (bigfont_gfx[(CharY + i)*bigset.Width + CharX+j] != 0){
				uint16 p = bigfont_gfx[(CharY + i)*bigset.Width + CharX+j] & color;
				GFX[(y + i + YOffset)*SCREEN_WIDTH + x + j + XOffset] = p|(1<<15);
			}
		}
	}
}

void printBigCenterString (uint16* GFX, int y, const char * str, unsigned int color){
	int l=0;
	for (unsigned int i=0;i<strlen(str);i++){
		if (str[i] == ' ') l += bigset.Base;
		else l += bigset.Chars[(unsigned short)str[i]].XAdvance;
	}
	printBigString (GFX, (SCREEN_WIDTH - l)/2, y, str, color);
}

void printBigString(uint16* GFX, int x, int y, const char * str, unsigned int color){
	int cursor = x;
	int ccursor = x;
	int line = y;
	for (unsigned int i=0;i<strlen(str);i++){
		if (str[i] == '\n'){
			line += bigset.LineHeight;
			cursor = x;
			ccursor = x;
			continue;
		}
		else if (str[i] == ' '){
			cursor += bigset.Base;
			clearRect(GFX, ccursor, line, ccursor + 11, line + bigset.LineHeight, bgColor);
			ccursor += 11;
			continue;
		}
		else if (cursor + bigset.Chars[(unsigned short)str[i]].XAdvance > SCREEN_WIDTH){
			line += bigset.LineHeight;
			cursor = x;
			ccursor = x;
		}
		if (ccursor < SCREEN_WIDTH){
			int end = ccursor + 11;
			if (end > SCREEN_WIDTH) end = SCREEN_WIDTH;
			clearRect(GFX, ccursor, line, end, line + bigset.LineHeight, bgColor);
		}
		ccursor += 11;
		printBigChar(GFX, cursor, line, str[i], color);
		cursor += bigset.Chars[(unsigned short)str[i]].XAdvance;
	}
}
