#include <cstring>

#include "topScreen.h"
#include "chessgfx.h"
#include "fonts.h"
#include "nds.h"
#include "san.h"
#include "move_do.h"

#define COMPUTER 1
#define ME 0
#define OTHERHUMAN 1

void refreshScreen (const bool repaint, const bool newMove, const int player, const bool isDebug,
		board_t *inputBoard, ChessClock *clockW, ChessClock *clockB, const search_best_t *SearchBest, 
		const int *moves, const undo_t *undos) {
	static uint32 oldW, oldB;
	static int oldBest, oldValue, oldDepth, oldPonder;
	char buf[128], buf1[16], buf2[16];
	setbgColor(BGCOLOR);
	if (repaint){
		swiWaitForVBlank();
		clearScreen (BG_GFX_SUB, BGCOLOR);
		printString(BG_GFX_SUB, 5, 5, 
			"DSChess 0.4 by Boris von Loesch\nEngine Fruit 2.1 by Fabien Letouzey", TEXT_COLOR, false);
	}
	uint32 wtime = clockW->getSeconds();
	if (repaint || wtime != oldW){
		if (repaint) printBigString(BG_GFX_SUB, 5, 30, "White", RGB15(20,20,20));
		printBigString(BG_GFX_SUB, 5, 42, clockW->toString(), RGB15(20,20,20));
		if (clockW->isFinished())
			printBigCenterString(BG_GFX_SUB, 100, "White has lost (time)!", RGB15(31,21,0));
	}
	uint32 btime = clockB->getSeconds();
	if (repaint || btime != oldB){
		if (repaint) printBigString(BG_GFX_SUB, 180, 30, "BLACK", TEXT_COLOR);
		printBigString(BG_GFX_SUB, 180, 42, clockB->toString(), TEXT_COLOR);
		if (clockB->isFinished())
			printBigCenterString(BG_GFX_SUB, 100, "Black has lost (time)!", RGB15(31,21,0));
	}
	oldW = wtime; oldB = btime;
	if (player == COMPUTER || isDebug){
		int move = SearchBest->move;
		int ponder = SearchBest->pv[1];
		if (repaint || move != oldBest || ponder != oldPonder){
	   		move_to_string(move, buf1, 256);
			if (isDebug){
		   		move_to_string(SearchBest->pv[1], buf2, 256);
				sprintf(buf, "Bestmove: %s,%s", buf1, buf2);
				printString(BG_GFX_SUB, 5, 70, buf, TEXT_COLOR);
			}
			else{
				sprintf(buf, "Bestmove: %s", buf1);
				printString(BG_GFX_SUB, 5, 70, buf, TEXT_COLOR);
			}
		}
		if (repaint || SearchBest->value != oldValue){
			sprintf(buf, "Rating: %d    ", SearchBest->value);
			printString(BG_GFX_SUB, 160, 70, buf, TEXT_COLOR);
		}
		if (repaint || SearchBest->depth != oldDepth){
			sprintf(buf, "Depth: %d ", SearchBest->depth);
			printString(BG_GFX_SUB, 160, 80, buf, TEXT_COLOR);
		}
		oldBest = move;
		oldValue = SearchBest->value;
		oldDepth = SearchBest->depth;
		oldPonder = ponder;
	}
	if (repaint || newMove){
		if (board_is_mate(inputBoard)){
			if (inputBoard->turn % 2 == 1) printBigCenterString(BG_GFX_SUB, 100, "Black is mate", RGB15(31,0,0));
			else printBigCenterString(BG_GFX_SUB, 100, "White is mate", RGB15(31,0,0));
		}			
		else if (board_is_check(inputBoard)){
			if (inputBoard->turn % 2 == 1) printBigCenterString(BG_GFX_SUB, 100, "Black is in check", RGB15(31,21,0));
			else printBigCenterString(BG_GFX_SUB, 100, "White is in check", RGB15(31,21,0));
		}
		else if (board_is_stalemate(inputBoard)){
			printBigCenterString(BG_GFX_SUB, 100, "Stalemate", TEXT_COLOR);
		}
		else if (isSecondRepetition(inputBoard)){
			printBigCenterString(BG_GFX_SUB, 100, "Stalemate (Rep)", TEXT_COLOR);
		}
		else clearRect(BG_GFX_SUB, 0, 100, SCREEN_WIDTH, 100+12, BGCOLOR);
		//newMove = false;
	}
}

void refreshScreen2 (bool repaint, const bool newMove, const int player, const bool isDebug,
		board_t *inputBoard, ChessClock *clockW, ChessClock *clockB, const search_best_t *SearchBest, 
		const int *moves, const undo_t *undos) {
	static uint32 oldW, oldB;
	char buf[16];
	setbgColor(BGCOLOR);
	const int maxCols = 2;
	const int maxRows = (SCREEN_HEIGHT-20)/10;
	
	if (repaint || newMove){
		swiWaitForVBlank();
		if (repaint) {
			clearScreen (BG_GFX_SUB, BGCOLOR);
			clearRect(BG_GFX_SUB, 0, 5+maxRows*FONTHEIGHT, SCREEN_WIDTH, SCREEN_HEIGHT, LIGHTGRAY);
		}
		const int l = inputBoard->sp;
		int show = l;
		if (show > 2*maxCols*maxRows){
			show = 2*maxCols*maxRows;
			if (!repaint) {
				clearScreen (BG_GFX_SUB, BGCOLOR);
				clearRect(BG_GFX_SUB, 0, 5+maxRows*FONTHEIGHT, SCREEN_WIDTH, SCREEN_HEIGHT, LIGHTGRAY);
				repaint = true;
			}
		}
		int start = inputBoard->sp - show;
		if (start % 2 == 1) start++;
		if (start + show > l) show--;
		for (int i=0; i<l - start; i++) {
			move_undo(inputBoard, moves[l-1-i], &undos[l-1-i]);
		}
		undo_t u[1];
		
		for (int i=0; i<show; i++) {
			int x = 5;
			if (i >= 2*maxRows) {
				x = SCREEN_WIDTH/2;
			}

			int y = (i/2) % maxRows;
			y *= FONTHEIGHT;
			y += 5;
			int add = 18;
			if (i % 2 == 1) {
				//black
				add = 75;
			}
			else {
				sprintf(buf, "%d.", (i+start)/2+1);
				printString(BG_GFX_SUB, x, y, buf, TEXT_COLOR);
			}
			
			getSan(buf, moves[start+i], inputBoard);
			printString(BG_GFX_SUB, x+add, y, buf, TEXT_COLOR, false);
			move_do(inputBoard, moves[start+i], u);
			//getSan(buf, inputBoard, moves, undos, start + i);
		}
	}
	setbgColor(LIGHTGRAY);
	uint32 wtime = clockW->getSeconds();
	if (repaint || wtime != oldW) {
		printString(BG_GFX_SUB, 5, 10+maxRows*FONTHEIGHT, clockW->toString(), RGB15(15,15,15));
	}
	uint32 btime = clockB->getSeconds();
	if (repaint || btime != oldB) {
		printString(BG_GFX_SUB, SCREEN_WIDTH-40, 10+maxRows*FONTHEIGHT, clockB->toString(), TEXT_COLOR);
	}
	oldW = wtime; oldB = btime;

}
